import {Component, Input} from '@angular/core'
import {TouchUpKit} from '../../shared/model/touch-up-kit.model'
import {formatDate, openTouchUpKit, reduceDashes} from '../../shared/utils/utils'
import {ErrorHandlingComponent} from '../../shared/components/subscribing.component'
import {TouchUpKitService} from '../service/touch-up-kit.service'
import {AccountService} from '../../accounts/services/account.service'
import {ErrorService} from '../../shared/service/error.service'
import {ActivatedRoute, Router} from '@angular/router'
import {Permission} from '../../shared/model/user'
import {RecipeService} from '../../colors/services/recipe.service'

@Component({
  selector: 'touchupkit-banner',
  templateUrl: 'banner.html',
  styles: [
    'p { margin-bottom: 0 }'
  ]
})
export class TouchUpKitBanner {
  @Input() touchUpKit: TouchUpKit

  get shippingDate(): string {
    return formatDate(this.touchUpKit.shippingDate)
  }

  get material(): string {
    return reduceDashes(this.touchUpKit.material)
  }
}

@Component({
  selector: 'touchupkit-list',
  templateUrl: './list.html'
})
export class TouchUpKitList extends ErrorHandlingComponent {
  touchUpKits$ = this.touchUpKitService.all
  columns = ['project', 'buggy', 'company', 'shippingDate', 'pdfButton', 'detailsButton', 'editButton']

  constructor(
    private touchUpKitService: TouchUpKitService,
    private accountService: AccountService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  openTouchUpKitPdf(touchUpKit: TouchUpKit) {
    openTouchUpKit(touchUpKit)
  }

  get canEditTouchUpKits(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_TOUCH_UP_KITS)
  }
}

@Component({
  selector: 'touchupkit-details',
  templateUrl: 'details.html'
})
export class TouchUpKitDetails extends ErrorHandlingComponent {
  touchUpKit: TouchUpKit | null
  contentTableCols = ['name', 'description', 'quantity']

  constructor(
    private touchUpKitService: TouchUpKitService,
    private recipeService: RecipeService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribeEntityById(
      this.touchUpKitService,
      this.urlUtils.parseIntUrlParam('id'),
      t => this.touchUpKit = t
    )
  }

  openPdf() {
    openTouchUpKit(this.touchUpKit)
  }
}

@Component({
  selector: 'touchupkit-add',
  templateUrl: 'add.html'
})
export class TouchUpKitAdd extends ErrorHandlingComponent {
  constructor(
    private touchUpKitService: TouchUpKitService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router,
  ) {
    super(errorService, activatedRoute, router)
  }

  submit(touchUpKit) {
    this.subscribeAndNavigate(
      this.touchUpKitService.save(touchUpKit),
      '/misc/touch-up-kit/list'
    )
  }
}

@Component({
  selector: 'touchupkit-edit',
  templateUrl: 'edit.html'
})
export class TouchUpKitEdit extends ErrorHandlingComponent {
  touchUpKit: TouchUpKit | null

  constructor(
    private touchUpKitService: TouchUpKitService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribeEntityById(
      this.touchUpKitService,
      this.urlUtils.parseIntUrlParam('id'),
      touchUpKit => this.touchUpKit = touchUpKit
    )
  }

  submit(touchUpKit) {
    this.subscribeAndNavigate(
      this.touchUpKitService.update(touchUpKit),
      '/misc/touch-up-kit/list'
    )
  }

  delete() {
    this.subscribeAndNavigate(
      this.touchUpKitService.delete(this.touchUpKit.id),
      '/misc/touch-up-kit/list'
    )
  }
}
